from ipykernel.kernelbase import Kernel
from pexpect import replwrap, EOF, TIMEOUT
import pexpect
import regex

from subprocess import PIPE, Popen
import signal
import re
import os

__version__ = '0.2.7'

class ACL2Kernel(Kernel):
    implementation = 'acl2_kernel'
    implementation_version = __version__

    @property
    def language_version(self):
        with Popen(os.environ['ACL2'], stdout=PIPE, stdin=PIPE) as p:
            p.stdin.close()
            return re.findall(r'ACL2 Version.*$', p.stdout.read().decode('utf-8'), re.MULTILINE)[0]
    
    @property
    def banner(self):
        return u'ACL2 Kernel (%s)' % self.language_version
    
    language_info = {
        'name': 'acl2',
        'codemirror_mode': 'commonlisp',
        'mimetype': 'text/x-common-lisp',
        'file_extension': '.lisp'
    }

    def __init__(self, **kwargs):
        Kernel.__init__(self, **kwargs)
        self._start_acl2()

    def _start_acl2(self):
        sig = signal.signal(signal.SIGINT, signal.SIG_DFL)
        try:
            prompt_change_cmd = '''
                    (encapsulate ()
                      (set-state-ok t)
                      (defun jupyter-prompt (channel state)
                        (declare (xargs :mode :program))
                        (fmt1 "JPY-ACL2>" '() 0 channel state nil)))
                    (defttag t)
                    (set-ld-prompt 'jupyter-prompt state)
                    (defttag nil)
                    (reset-prehistory)'''
            self.acl2wrapper = replwrap.REPLWrapper(os.environ['ACL2'], 'ACL2 !>', prompt_change_cmd, 'JPY-ACL2>')
            # Discard the output of prompt_change_cmd.
            for i in range(3):
                self.acl2wrapper.run_command(';', timeout=None)
        finally:
            signal.signal(signal.SIGINT, sig)
        
    def do_execute(self, code, silent, store_history=True, user_expressions=None, allow_stdin=False):
        if not code.strip():
            return {
                'status': 'ok',
                'execution_count': self.execution_count,
                'payload': [],
                'user_expressions': []
            }
        interrupted = False
        try:
            # Remove all comments from the code block.
            cmd = re.sub(r';.*$', '', code, flags=re.MULTILINE)

            # Count all commands in the code block. There are 2 types of
            # commands recognized:
            #
            # 1. Keyword commands, such as:
            #      :pl subsetp
            #    using the ^[ \t]*:.*$ regex. This is a simple recognizer that
            #    treats the whole line as the keyword command.
            # 2. Top-level LISP forms, such as:
            #      (defun app (x y)
            #        (cond ((endp x) y)
            #              (t (cons (car x)
            #                       (app (cdr x) y)))))
            #    using the \((?>[^()]|(?R))*\) regex.
            num_cmds = len(regex.findall(r'^[ \t]*:.*$|\((?>[^()]|(?R))*\)', cmd, regex.MULTILINE))

            # Convert all \r and \n into spaces.
            cmd = re.sub(r'[\r\n]', ' ', cmd.strip())

            output = self.acl2wrapper.run_command(cmd, timeout=None)
            for i in range(num_cmds - 1):
                output += self.acl2wrapper.run_command(';', timeout=None)
            while True:
                try:
                    more_output = self.acl2wrapper.run_command(';', timeout=0)
                except TIMEOUT:
                    break
                output += more_output
        except KeyboardInterrupt:
            self.acl2wrapper.child.sendintr()
            interrupted = True
            self.acl2wrapper._expect_prompt()
            output =  self.acl2wrapper.child.before
            self.process_output(output)
        except EOF:
            output = self.acl2wrapper.child.before + 'Restarting ACL2'
            self._start_acl2()
            self.process_output(output)
        if interrupted:
            return {
                'status': 'abort',
                'execution_count': self.execution_count
            }
        if not silent:
            stream_content = {
                'execution_count': self.execution_count,
                'name': 'stdout',
                'text': output
            }
            self.send_response(self.iopub_socket, 'stream', stream_content)
        return {
            'status': 'ok',
            'execution_count': self.execution_count,
            'payload': [],
            'user_expressions': {}
        }



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['acl2_kernel']

package_data = \
{'': ['*']}

install_requires = \
['ipykernel>=5.3.0,<6.0.0',
 'ipython>=7.15.0,<8.0.0',
 'jupyter_client>=6.1.3,<7.0.0',
 'pexpect>=4.8.0,<5.0.0',
 'regex>=2021.4.4,<2022.0.0']

setup_kwargs = {
    'name': 'acl2-kernel',
    'version': '0.2.7',
    'description': 'Jupyter Kernel for ACL2',
    'long_description': '# acl2-kernel [![PyPI](https://img.shields.io/pypi/v/acl2-kernel)](https://pypi.org/project/acl2-kernel/) [![Binder](https://mybinder.org/badge_logo.svg)](https://mybinder.org/v2/gh/tani/acl2-kernel/master?filepath=Example.ipynb)\n\nJupyter Kernel for ACL2\n\n## What is Jupyter and ACL2?\n\n> Project Jupyter exists to develop open-source software, open-standards, and services for interactive computing across dozens of programming languages. (https://jupyter.org/)\n\n> ACL2 is a logic and programming language in which you can model computer systems, together with a tool to help you prove properties of those models. "ACL2" denotes "A Computational Logic for Applicative Common Lisp". (http://www.cs.utexas.edu/users/moore/acl2/)\n\n## Usage\n\nWe follow to the standard jupyter kernel installation. So, you will install the kernel by `pip` command,\nand will call the installation command like,\n\n```sh\n$ pip3 install jupyter acl2-kernel\n$ python3 -m acl2_kernel.install\n$ jupyter noteboook\n```\n\nYou also can see the deep usage by `python3 -m acl2_kernel.install --help`.\n\n### Docker \n\nIn some case, you might want to run the kernel in the Docker containers.\nThis repository contains Dockerfile example. You can build example image by the following command.\n\n```\n$ docker build . -t acl2\n```\n\nTo run the container, you would type the command like\n\n```\n$ docker run --rm -p 8888:8888 acl2 jupyter notebook --ip=\'0.0.0.0\'\n```\n\nA running example is available in the `example/` directory.\nYou can try it on [![Binder](https://mybinder.org/badge_logo.svg)](https://mybinder.org/v2/gh/tani/acl2-kernel/master?filepath=Example.ipynb).\n\n## Related Projects\n\n- [Jupyter](https://jupyter.org/) - Softwares for interactive computing\n- [ACL2](http://www.cs.utexas.edu/users/moore/acl2/) - Theorem prover based on Common Lisp\n\n## License\n\nThis project is released under the BSD 3-clause license.\n\nCopyright (c) 2020, TANIGUCHI Masaya All rights reserved.\n\nWe borrow code from the following projects.\n\n- Egison Kernel; Copyright (c) 2017, Satoshi Egi and contributors All rights reserved.\n- Bash Kernel; Copyright (c) 2015, Thomas Kluyver and contributors All rights reserved.\n',
    'author': 'TANIGUCHI Masaya',
    'author_email': 'm@docs.casa',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/tani/acl2-kernel',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)
